<?php
/**
 * Plugin Name: CDN Bull
 * Plugin URI: https://cdnbull.com
 * Description: Simply integrate a Content Delivery Network (CDN) into your WordPress sites. Host and optimize image, CSS, and JavaScript files with Statically CDN.
 * Version: 1.0.3
 * Author: cdnbull
 * Author URI: https://cdnbull.com/#contact
 * License: GPL2+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.txt
 * Text Domain: cdn-bull

 */

/*
   Copyright (C)  2021 CDN Bull

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// constants
define( 'CDNBULL_VERSION', '1.0.3');
define( 'CDNBULL_MIN_PHP', '7.0' );
define( 'CDNBULL_MIN_WP', '5.1' );
define( 'CDNBULL_FILE', __FILE__ );
define( 'CDNBULL_BASE', plugin_basename( __FILE__ ) );
define( 'CDNBULL_DIR', __DIR__ );
define( 'CDNBULL_PLUGIN_SLUG', 'cdn-bull' );
define( 'CDNBULL_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
define( 'CDNBULL_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
define( 'CDNBULL_URL', 'https://cdnbull.com/' );
define( 'CDNBULL_API_URL', 'https://api.cdnbull.com/' );
define( 'CDNBULL_GLOBAL_EXCLUDES', '.php,.zip,LICENSE,wp-admin,cgi-bin,.DS_Store,.github,.git' );

// hooks
add_action( 'plugins_loaded', array( 'CDNBULL', 'init' ) );
register_activation_hook( __FILE__, array( 'CDNBULL', 'on_activation' ) );
register_uninstall_hook( __FILE__, array( 'CDNBULL', 'on_uninstall' ) );

// register autoload
spl_autoload_register( 'cdnbull_autoload' );

// load required classes
function cdnbull_autoload( $class_name ) {
    if ( in_array( $class_name, array( 'CDNBULL', 'CDNBULL_Engine', 'CDNBULL_Settings', 'CDNBULL_Upload', 'CDNBULL_Purge', 'CDNBULL_HM_Backup', 'CDNBULL_Backups' ) ) ) {
        require_once sprintf(
            '%s/inc/%s.class.php',
            CDNBULL_DIR,
            strtolower( $class_name )
        );
    }
}