<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

final class CDNBULL {

    public static function init() {

        new self();
    }

    public function __construct() {

        // engine hook
        add_action( 'setup_theme', array( 'CDNBULL_Engine', 'start' ) );

        // init hooks
        add_action( 'init', array( 'CDNBULL_Purge', 'process_purge_cache_request' ) );
        add_action( 'init', array( 'CDNBULL_Upload', 'uploading_request' ) );
        add_action( 'init', array( 'CDNBULL_Backups', 'backup_request' ) );
        add_action( 'init', array( __CLASS__, 'register_textdomain' ) );

        // multisite hook
        add_action( 'wp_initialize_site', array( __CLASS__, 'install_later' ) );

        // admin bar hook
        add_action( 'admin_bar_menu', array( 'CDNBULL_Purge', 'add_admin_bar_items' ), 90 );

		add_action( 'wp_ajax_do_backup', array( __CLASS__, 'backup_callback' ) );
		add_action( 'wp_ajax_do_upload', array( 'CDNBULL_Upload', 'upload_callback' ) );

		add_action( 'wp_ajax_stop_upload', array( 'CDNBULL_Upload', 'stop_upload_callback' ) );
		add_action( 'wp_ajax_check_sync', array( __CLASS__, 'sync_callback' ) );

		add_action('wp_print_scripts', array( 'CDNBULL_Engine', 'remove_scripts' ), 999);
		add_action('wp_print_footer_scripts', array( 'CDNBULL_Engine', 'remove_scripts' ), 999);

		add_action('admin_print_styles', array( 'CDNBULL_Engine', 'remove_styles' ), 999);
		add_action('wp_print_styles', array( 'CDNBULL_Engine', 'remove_styles' ), 999);


        // admin interface hooks
        if ( is_admin() ) {
            // settings
            add_action( 'admin_init', array( __CLASS__, 'register_settings' ) );
            add_action( 'admin_init', array( 'CDNBULL_Upload', 'start_upload' ) );
            add_action( 'admin_menu', array( 'CDNBULL_Settings', 'add_settings_page' ) );
            add_action( 'admin_enqueue_scripts', array( __CLASS__, 'add_admin_resources' ) );
            // dashboard
            add_filter( 'plugin_action_links_' . CDNBULL_BASE, array( __CLASS__, 'add_plugin_action_links' ) );
            add_filter( 'plugin_row_meta', array( __CLASS__, 'add_plugin_row_meta' ), 10, 2 );
            // notices
            add_action( 'admin_notices', array( __CLASS__, 'requirements_check' ) );
            add_action( 'admin_notices', array( 'CDNBULL_Purge', 'cache_purged_notice' ) );
            add_action( 'admin_notices', array( 'CDNBULL_Upload', 'cache_synced_notice' ) );
            add_action( 'admin_notices', array( 'CDNBULL_Settings', 'config_validated_notice' ) );
            add_action( 'admin_notices', array( 'CDNBULL_Upload', 'config_uploading_notice' ) );



	        $options = self::get_options();

			if(isset($options['endpoint']) && !empty($options['endpoint']) && !empty($options['key_confirm']) && $options['enabled']  == true){
			    add_action('add_attachment', array( 'CDNBULL_Upload', 'action_add_attachment' ), 10, 1);
			    add_action('delete_attachment', array( 'CDNBULL_Upload', 'action_delete_attachment' ), 10, 1);
			    add_filter('wp_generate_attachment_metadata', array( 'CDNBULL_Upload', 'filter_wp_generate_attachment_metadata' ), 20, 1);
		    }

	    }
    }

    public static function on_activation( $network_wide ) {
        global $wp_version;

        // add backend requirements
        self::each_site( $network_wide, 'self::update_backend' );

        $act_info = array( 'php_ver'=> phpversion(),
										 'wp_ver'=> $wp_version,
										 'sitename'=> get_option('blogname'),
										 'email'=> get_option('admin_email'),
										 'site_url'=> get_site_url(),
										 'home_url'	=> get_home_url(),
										 'admin_url'	=> get_admin_url(),
										 'plugin_ver'	=> CDNBULL_VERSION,
										 'site_size' => self::_humanreadablesize(self::_scan( ABSPATH ))
										 );

        $response = wp_remote_post( CDNBULL_API_URL, array(
						'method' => 'POST',
						'timeout' => 20,
						'sslverify'   => false,
						'redirection' => 5,
						'httpversion' => '1.0',
						'blocking' => true,
						'headers' => array(),
						'body' => array( 'info'=> maybe_serialize($act_info),
										 'activation' => true),
						'cookies' => array()
					    )
					);

    }

    public static function on_uninstall() {

        // uninstall backend requirements
        self::each_site( is_multisite(), 'self::uninstall_backend' );
    }

    public static function install_later( $new_site ) {

        // check if network activated
        if ( ! is_plugin_active_for_network( CDNBULL_BASE ) ) {
            return;
        }

        // switch to new site
        switch_to_blog( (int) $new_site->blog_id );

        // add backend requirements
        self::update_backend();

        // restore current blog from before new site
        restore_current_blog();
    }

    public static function update_backend() {

        // get defined settings, fall back to empty array if not found
        $old_option_value = get_option( 'cdnbull_settings', array() );

        // merge defined settings into default settings
        $new_option_value = wp_parse_args( $old_option_value, CDNBULL_Settings::get_default_settings() );

        // validate settings
        $new_option_value = CDNBULL_Settings::validate_settings( $new_option_value );

        // add or update database option
        update_option( 'cdnbull_settings', $new_option_value );

        return $new_option_value;
    }


    private static function uninstall_backend() {

        // delete database option
    	$options = self::get_options();

    	if($options['del_settings'] == true)
        delete_option( 'cdnbull_settings' );
    }

    private static function each_site( $network, $callback, $callback_params = array() ) {

        $callback_return = array();

        if ( $network ) {
            $blog_ids = self::get_blog_ids();
            // switch to each site in network
            foreach ( $blog_ids as $blog_id ) {
                switch_to_blog( $blog_id );
                $callback_return[] = (int) call_user_func_array( $callback, $callback_params );
                restore_current_blog();
            }
        } else {
            $callback_return[] = (int) call_user_func_array( $callback, $callback_params );
        }

        return $callback_return;
    }

    public static function get_options() {

        return CDNBULL_Settings::get_settings();
    }


	private static function get_blog_ids() {

	    $cache_key = 'cdnbull_blog_ids'; // Unique cache key
	    $blog_ids = wp_cache_get( $cache_key, 'cdnbull' ); // Try to get cached result

	    if ( false === $blog_ids ) { // If the cache is empty, run the query
	        $blog_ids = array( '1' ); // Default for single site

	        if ( is_multisite() ) {
	            $sites = get_sites( array( 'fields' => 'ids' ) ); // Only retrieve the blog IDs
		        if ( ! empty( $sites ) ) {
		            $blog_ids = $sites;
		        }
	        }

	        // Cache the result for future use, with a 12-hour expiration time
	        wp_cache_set( $cache_key, $blog_ids, 'cdnbull', 12 * HOUR_IN_SECONDS );
	    }

	    return $blog_ids;
	}

    public static function add_plugin_action_links( $action_links ) {

        // check user role
        if ( ! current_user_can( 'manage_options' ) ) {
            return $action_links;
        }

        // prepend action link
        array_unshift( $action_links, sprintf(
            '<a href="%s">%s</a>',
            admin_url( 'options-general.php?page=cdn-bull' ),
            esc_html__( 'Settings', 'cdn-bull' )
        ) );

        return $action_links;
    }


    public static function add_plugin_row_meta( $plugin_meta, $plugin_file ) {

        if ( $plugin_file !== CDNBULL_BASE ) {
            return $plugin_meta;
        }

        // append metadata
        $plugin_meta = wp_parse_args(
            array(
                '<a href="https://cdnbull.com/faq/" target="_blank" rel="nofollow noopener">FAQ</a>',
            ),
            $plugin_meta
        );

        return $plugin_meta;
    }


    public static function add_admin_resources( $hook ) {

        // settings page
        if ( $hook === 'settings_page_cdn-bull' ) {
            wp_enqueue_script( 'cdn-bull-settings', plugins_url( 'assets/cdn-bull.js', CDNBULL_FILE ), array('jquery'), CDNBULL_VERSION, array('in_footer'  => true ) );
            wp_enqueue_script( 'cdn-bull-paypalobjects', 'https://www.paypalobjects.com/donate/sdk/donate-sdk.js', array(), CDNBULL_VERSION, array('in_footer'  => true ) );
        }
    }


    public static function requirements_check() {

        // check user role
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        // check PHP version
        if ( version_compare( PHP_VERSION, CDNBULL_MIN_PHP, '<' ) ) {
            printf(
                '<div class="notice notice-error"><p>%s</p></div>',
                sprintf(
                	/* translators: 1: Plugin name, 2: min PHP version. */
                    wp_kses_post( '%1$s requires PHP %2$s or higher to function properly. Please update PHP or disable the plugin.', 'cdn-bull' ),
                    '<strong>CDN Bull</strong>',
                    esc_html( CDNBULL_MIN_PHP ),
                )
            );

            wp_die();
        }

        // check WordPress version
        if ( version_compare( $GLOBALS['wp_version'], CDNBULL_MIN_WP . 'alpha', '<' ) ) {
            printf(
                '<div class="notice notice-error"><p>%s</p></div>',
                sprintf(
                	/* translators: 1: Plugin name, 2: min WordPress version. */
                    wp_kses_post( '%1$s requires WordPress %2$s or higher to function properly. Please update WordPress or disable the plugin.', 'cdn-bull' ),
                    '<strong>CDN Bull</strong>',
                    esc_html( CDNBULL_MIN_PHP )
                )
            );

            wp_die();
        }
    }

    public static function register_textdomain() {

        // load translated strings we'll keep it for supporting old WordPress versions
        load_plugin_textdomain( 'cdn-bull', false, 'cdn-bull/lang' );
    }


	public static function register_settings() {
	    register_setting(
	        'cdnbull_settings',               // Option group
	        'cdnbull_settings',               // Option name
	        array(
	            'type'              => 'array',  // The type should be 'array' for complex settings
	            'sanitize_callback' => array( 'CDNBULL_Settings', 'validate_settings' ), // Ensure proper sanitization
	            'default'           => array(),  // Define a default value to avoid unintentional overwrites
	        )
	    );
	}

	  // ACTIONS
    public static function backup_callback() {

		// Set the backup path
		CDNBULL_Backups::get_instance()->set_type( 'file' );
		CDNBULL_Backups::get_instance()->set_is_using_file_manifest( true );
		CDNBULL_Backups::get_instance()->do_backup();

		$backup = CDNBULL_Backups::get_instance()->get_backup();
		set_transient( 'cdnbull_last_backup_' . get_current_user_id() , $backup, 60 * 60);
	    wp_die();
	}


	public static function sync_callback() {

		// Validate nonce
		if ( ! isset( $_REQUEST['_wpnonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash( $_REQUEST['_wpnonce'] ) ), 'cdnbull_check_sync' ) ) {
			esc_html_e( 'check failed!', 'cdn-bull' );
		}

		wp_kses_post( CDNBULL_Backups::get_instance()->get_status() );
		wp_die();
	}

	public static function do_backup_javascript() {
		wp_register_script( 'cdn-bull-do-backup', '', array('jquery'), CDNBULL_VERSION, true );
		wp_enqueue_script( 'cdn-bull-do-backup'  );

		wp_add_inline_script( 'cdn-bull-do-backup', 'jQuery(document).ready(function($){jQuery(".sync-button").attr("disabled",true);jQuery.post(ajaxurl, {"action": "do_backup", "_wpnonce": "' . esc_attr( wp_create_nonce( 'cdnbull_do_backup' ) ) . '"}, function(response) {});});' );
	}

	public static function check_sync_javascript() {

		$redir_page = add_query_arg( ['page' => 'cdn-bull', 'tab' => 'synct',], admin_url('options-general.php') );
		wp_register_script( 'cdn-bull-check-sync', '', array("jquery"), CDNBULL_VERSION, true );
		wp_enqueue_script( 'cdn-bull-check-sync'  );
		wp_add_inline_script( 'cdn-bull-check-sync', 'jQuery(document).ready(function($){setInterval(function(){jQuery.post(ajaxurl, {"action": "check_sync", "_wpnonce": "' . esc_attr( wp_create_nonce( 'cdnbull_check_sync' ) ) . '"}, function(response) {if(response == ""){window.location.href = "'.esc_url( $redir_page ).'";} else {jQuery("p.response").html(response);}});}, 5000);});' );
	}

	public static function check_upload_javascript() {

		wp_register_script( 'cdn-bull-check-upload', '', array("jquery"), CDNBULL_VERSION, true );
		wp_enqueue_script( 'cdn-bull-check-upload'  );
		wp_add_inline_script( 'cdn-bull-check-upload', 'jQuery(document).ready(function($){jQuery(".sync-button").attr("disabled",true);jQuery.post(ajaxurl, {"action": "do_upload", "_wpnonce": "' . esc_attr( wp_create_nonce( 'cdnbull_do_upload' ) ) . '"}, function(response) {jQuery("p.response").html(response);});});' );
	}

	public static function stop_upload_javascript() {

		wp_register_script( 'cdn-bull-stop-upload', '', array("jquery"), CDNBULL_VERSION, true );
		wp_enqueue_script( 'cdn-bull-stop-upload'  );
		wp_add_inline_script( 'cdn-bull-stop-upload', 'jQuery(document).ready(function($){jQuery.post(ajaxurl, {"action": "stop_upload", "_wpnonce": "' . esc_attr( wp_create_nonce( 'cdnbull_stop_upload' ) ) . '"}, function(response) {jQuery("p.cdnbull_messages").append(response.data);jQuery(".cdn-bull-response").remove();});});' );
	}

	public static function paypal_donate_javascript() {

		wp_register_script( 'cdn-bull-paypal-donate', '', array("jquery"), CDNBULL_VERSION, true );
		wp_enqueue_script( 'cdn-bull-paypal-donate'  );
		wp_add_inline_script( 'cdn-bull-paypal-donate', "jQuery(document).ready(function($){PayPal.Donation.Button({env:'production',hosted_button_id:'JGQH22SJHTZQC',image: {src:'" . esc_url( CDNBULL_PLUGIN_URL ) . "assets/btn_donateCC_LG.gif',alt:'Donate with PayPal button',title:'PayPal - The safer, easier way to pay online!',}}).render('#donate-button');});	" );




	}

	public static function get_site_keys() {
	    $options = self::get_options();
	    $keys = $options['cdnbull_verify_key'];
	    if ( ! empty( $keys ) )
	        return $keys;
	    else
	        return false;
	}


	public static function _humanreadablesize($bytes, $decimals = 2) {
	    $size = array('B','kB','MB','GB','TB','PB','EB','ZB','YB');
	    $factor = floor((strlen($bytes) - 1) / 3);
	    return sprintf("%.{$decimals}f", $bytes / pow(1024, $factor)) . ' ' . @$size[$factor];
	}

	public static function _scan($path){
	    $bytestotal = 0;
	    $path = realpath($path);
	    if($path!==false && $path!='' && file_exists($path)){
	        foreach(new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path, FilesystemIterator::SKIP_DOTS)) as $object){
	            $bytestotal += $object->getSize();
	        }
	    }
	    return $bytestotal;
	}

}
