<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

final class CDNBULL_Settings {

    public static function init() {

        new self();
    }

    public function __construct() {

    }

    private static function get_config_validated_transient_name() {

        $transient_name = 'cdnbull_config_validated_' . get_current_user_id();

        return $transient_name;
    }

    private static function validate_config( $validated_settings ) {

        if (!isset($validated_settings['cdnbull_verify_key'])) {
            $validated_settings['cdnbull_verify_key'] = "";
            $validated_settings['endpoint'] = "";
            $validated_settings['key_confirm'] = "";
            $validated_settings['enabled'] = 0;
            $validated_settings['last_error'] = "No key please add key";
        } else {
        	$check_key = self::confirm_key($validated_settings['cdnbull_verify_key']);

        	if($check_key['endpoint'] != '' && $check_key['key'] == true){
	        	$validated_settings['endpoint'] = $check_key['endpoint'];
	        	$validated_settings['key_confirm'] = $check_key['key'];
	        	$validated_settings['last_error'] = "";
        	} else {
	        	$validated_settings['endpoint'] = "";
	        	$validated_settings['key_confirm'] = "";
	        	$validated_settings['last_error'] = $check_key['last_error'];
	        	$validated_settings['enabled'] = 0;
        	}
        }

        if ( empty( $validated_settings['endpoint'] ) ) {
            return $validated_settings;
        }

        // set transient for config validation notice
        set_transient( self::get_config_validated_transient_name(), $check_key );

        // validate config
        if ( !empty( $response['last_error']) ) {
            $validated_settings['endpoint'] = '';
        }

        return $validated_settings;
    }

    public static function config_validated_notice() {

        // check user role
        if ( ! current_user_can( 'manage_options' ) ) {
            return;
        }

        $response = get_transient( self::get_config_validated_transient_name() );

        if ( isset($response['last_error']) && !empty($response['last_error']) ) {
            printf(
                esc_html( $response['last_error'] )
            );

            delete_transient( self::get_config_validated_transient_name() );
        }
    }

    public static function get_settings() {

        // get database option value
        $settings = get_option( 'cdnbull_settings' );

        // if database option does not exist or settings are outdated
        if ( $settings === false || ! isset( $settings['version'] ) || $settings['version'] !== CDNBULL_VERSION ) {
            $settings = CDNBULL::update_backend();
        }

        return $settings;
    }

    public static function get_default_settings( $settings_type = null ) {

        $system_default_settings = array(
        	'version' => (string) CDNBULL_VERSION,
        	'endpoint'             	   => '',
            'key_confirm'              => '',
            'cdnbull_verify_key'        => '',
            'del_settings'  		   => '0',
            'rewrite'        		   => '1',
            'enabled'        		   => '0',
            'last_error'        	   => '',
        );

        if ( $settings_type === 'system' ) {
            return $system_default_settings;
        }

        $user_default_settings = array(
            'included_file_extensions' => implode( PHP_EOL, array(
                                              '.avif',
                                              '.css',
                                              '.gif',
                                              '.jpeg',
                                              '.jpg',
                                              '.json',
                                              '.js',
                                              '.mp3',
                                              '.mp4',
                                              '.pdf',
                                              '.png',
                                              '.svg',
                                              '.webp',
                                              '.woff',
                                              '.woff2',
                                              '.tff',
                                          ) ),
            'excluded_strings'         => implode( PHP_EOL, array(
                                              '.php',
                                          ) ),
        );

        // merge default settings
        $default_settings = wp_parse_args( $user_default_settings, $system_default_settings );

        return $default_settings;
    }


    private static function convert_settings( $settings ) {

        // check if there are any settings to convert
        if ( empty( $settings ) ) {
            return $settings;
        }

        // updated settings
        if ( isset( $settings['url'] ) && is_string( $settings['url'] ) && substr_count( $settings['url'], '/' ) > 2 ) {
            $settings['url'] = '';
        }

        // reformatted settings
        if ( isset( $settings['excludes'] ) && is_string( $settings['excludes'] ) ) {
            $settings['excludes'] = str_replace( ',', PHP_EOL, $settings['excludes'] );
            $settings['excludes'] = str_replace( '.php', '', $settings['excludes'] );
        }

        // renamed or removed settings
        $settings_names = array(
            'dirs'     => '', // deprecated
            'url'      => '', // deprecated
            'excludes' => 'excluded_strings',
            'relative' => '', // deprecated
            'https'    => '', // deprecated
        );

        foreach ( $settings_names as $old_name => $new_name ) {
            if ( array_key_exists( $old_name, $settings ) ) {
                if ( ! empty( $new_name ) ) {
                    $settings[ $new_name ] = $settings[ $old_name ];
                }

                unset( $settings[ $old_name ] );
            }
        }

        return $settings;
    }


    public static function confirm_key($key) {

        // validate nonce
        if ( empty($key)) {
            return;
        }

        // check user role
        if ( ! is_admin() ) {
            return;
        }

        // load if network
        if ( ! function_exists('is_plugin_active_for_network') ) {
            require_once( ABSPATH. 'wp-admin/includes/plugin.php' );
        }

        $response = wp_remote_post( CDNBULL_API_URL, array(
						'method' => 'POST',
						'timeout' => 20,
						'sslverify'   => false,
						'redirection' => 5,
						'httpversion' => '1.0',
						'blocking' => true,
						'headers' => array(),
						'body' => array( 'sk'=> $key,
										 'sitename'=> get_site_url(),
										 'verify' => true),
						'cookies' => array()
					    )
					);

        // check results - error connecting
        if ( is_wp_error( $response ) ) {

        	return array( 'last_error' => sprintf(
        		/* translators: 1: Error response text from server*/
	            esc_html__('Error connecting to API - %s', 'cdn-bull' ),
	            $response->get_error_message()
	        ) );
        }

        // check HTTP response
        if ( is_array( $response ) and is_admin() ) {
            $json = json_decode($response['body'], true);

            $rc = wp_remote_retrieve_response_code( $response );

            // success
            if ( $rc == 200
                    and is_array($json)
                    and array_key_exists('endpoint', $json)
                    and array_key_exists('key', $json) )
            {
                return $json;
            } elseif ( is_array($json)
                    and array_key_exists('last_error', $json)
                    and $json['last_error'] != "" ) {
            	return array('last_error' => $json['last_error']);
            }elseif ( $rc == 200 ) {
                // return code 200 but no message
                return array('last_error' => esc_html__('HTTP returned 200 but no message received.', 'cdn-bull' ));
            } else {
                // Something else went wrong - show HTTP error code

	        	return array( 'last_error' => sprintf(
	        		 /* translators: 1:  HTTP error code from server*/
		            esc_html__('HTTP returned - %s', 'cdn-bull' ),
		            $rc
		        ) );
            }
        }
    }

    public static function validate_settings( $settings ) {
    // Update option for the upload URL path
        update_option('upload_url_path', "");

    // Validate settings with sanitization
        $validated_settings = array(
            'included_file_extensions'	=> self::validate_textarea( $settings['included_file_extensions'], true ),
            'excluded_strings'			=> self::validate_textarea( $settings['excluded_strings'] ),
            'cdnbull_verify_key'		=> (string) sanitize_text_field( $settings['cdnbull_verify_key'] ),
            'enabled'           		=> (string) sanitize_text_field( $settings['enabled'] ),
        	'endpoint'                 	=> esc_url_raw( $settings['endpoint'] ),  // Ensure URL is sanitized properly
            'key_confirm'           	=> (string) sanitize_text_field( $settings['key_confirm'] ),
            'rewrite'           		=> isset($settings['rewrite']) ? (string) sanitize_text_field( $settings['rewrite'] ) : 0,
            'del_settings'           	=> isset($settings['del_settings']) ? (string) sanitize_text_field( $settings['del_settings'] ) : 0 ,
            'last_error'           		=> (string) sanitize_text_field( $settings['last_error'] ),
        );

    // Add default system settings
        $validated_settings = wp_parse_args( $validated_settings, self::get_default_settings( 'system' ) );

    // Check if configuration should be validated
        if ( ! empty( $settings['validate_config'] ) ) {
            $validated_settings = self::validate_config( $validated_settings );
        }

        return $validated_settings;
    }

    private static function validate_textarea( $textarea, $file_extension = false ) {

        $textarea = sanitize_textarea_field( $textarea );
        $lines = explode( PHP_EOL, trim( $textarea ) );
        $validated_textarea = array();

        foreach ( $lines as $line ) {
            $line = trim( $line );

            if ( $line !== '' ) {
                if ( ! $file_extension ) {
                    $validated_textarea[] = $line;
                } elseif ( preg_match( '/^\.\w{1,10}$/', $line ) ) {
                    $validated_textarea[] = $line;
                }
            }
        }

        $validated_textarea = implode( PHP_EOL, $validated_textarea );

        return $validated_textarea;
    }

    public static function add_settings_page() {

        add_options_page(
            'CDN Bull',
            'CDN Bull',
            'manage_options',
            CDNBULL_PLUGIN_SLUG,
            array( __CLASS__, 'settings_page' )
        );
    }

	private static function page_tabs( $current = 'settings' ) {
	    $tabs = array(
	        'settings'   => esc_html__( 'Settings', 'cdn-bull' ),
	        'synct'   => esc_html__( 'Sync', 'cdn-bull' ),
	        'support'  => esc_html__( 'Support', 'cdn-bull' ),
	        'donate'  => esc_html__( 'Donate', 'cdn-bull' ),
	        'hire_expert'  => esc_html__( 'Hire an Expert', 'cdn-bull' ),
	    );

	    $html = '<div class="wrap"><h1></h1>';

	    $html .=  sprintf(
		    			'<a href="%s" target="_self">%s</a>',
		    			add_query_arg( ['page' => CDNBULL_PLUGIN_SLUG,], admin_url('options-general.php') ),
		    			'<img style="padding-top:20px" src="' . esc_url( CDNBULL_PLUGIN_URL.'assets/logo.png' ) . '" width="300" alt="">'
	    			);

	    $html .= '<h2 class="nav-tab-wrapper cdnbull_settings">';
	    foreach( $tabs as $tab => $name ){
	        $class = ( $tab == $current ) ? 'nav-tab-active' : '';
	        $html .= sprintf( '<a href="%s" class="' . $class . ' nav-tab">%s</a>',
					        		add_query_arg(
					        			['page' => CDNBULL_PLUGIN_SLUG,
					        			'tab' => $tab],
					        			admin_url('options-general.php')
					        		),
					        	$name );
	    }

	    $html .= '</h2>';
	    return $html;
	}


    public static function settings_page() {

		$options = CDNBULL::get_options();
		// Tabs
		$tab = ( ! empty( $_GET['tab'] ) ) ? sanitize_text_field( wp_unslash( $_GET['tab'] ) ) : 'settings';
        echo wp_kses_post( self::page_tabs( $tab ));


        if ( $tab == 'settings' ) { ?>
            <h1><?php esc_html_e( 'CDN Bull Settings', 'cdn-bull' ); ?></h1>
            <form method="post" action="options.php">
                <?php settings_fields('cdnbull_settings') ?>
                <table class="form-table">
	                   <tr valign="top">
	                       <th scope="row">
	                           <?php esc_html_e("Enable CDN", "cdn-bull"); ?>
	                       </th>
	                       <td>
	                           <fieldset>
	                               <label for="cdnbull_enabled">
	                                   <input type="checkbox" name="cdnbull_settings[enabled]" id="settings_enabled" value="1" <?php checked(1, esc_html( $options['enabled'] )) ?> <?php
	                                   if(isset($options['endpoint']) && empty($options['endpoint']) && empty($options['key_confirm'])	){
	                                   		echo ' disabled';
	                                   }
	                                   ?>/>
	                                   <?php esc_html_e("Enable CDN", "cdn-bull"); ?>
	                               </label>
	                           </fieldset>
	                       </td>
	                   </tr>
	                   <tr valign="top">
	                       <th scope="row">
	                           <?php esc_html_e("Enable rewrite", "cdn-bull"); ?>
	                       </th>
	                       <td>
	                           <fieldset>
	                               <label for="cdnbull_rewrite">
	                                   <input type="checkbox" name="cdnbull_settings[rewrite]" id="cdnbull_rewrite" value="1" <?php checked(1, esc_html( $options['rewrite'] )) ?> />
	                                   <?php esc_html_e("Enable rewrite for the uploaded files (default: enabled).", "cdn-bull"); ?>
	                               </label>
										<?php
											if(isset($options['endpoint']) && !empty($options['endpoint']) && !empty($options['key_confirm'])	){
												echo '<p>
														 CDN endpoint : <a href="javascript:void(0)" data-text="'.esc_html( $options['endpoint'].$options['cdnbull_verify_key'] ).'" onclick="copyClipboard(\'1\')" class="site-name-1">'.esc_html( $options['endpoint'].$options['cdnbull_verify_key'] ).'/</a>
														 <span style="display:none" class="copied-1">copied!!</span>
													  </p>';
											}
										?>
	                           </fieldset>
	                       </td>
	                   </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php esc_html_e( 'CDN Inclusions', 'cdn-bull' ); ?>
                        </th>
                        <td>
                            <fieldset>
                                <p class="subheading"><?php esc_html_e( 'File Extensions', 'cdn-bull' ); ?></p>
                                <label for="cdnbull_included_file_extensions">
                                    <textarea name="cdnbull_settings[included_file_extensions]" type="text" id="cdnbull__included_file_extensions" class="regular-text code" rows="5" cols="40"><?php echo esc_textarea( CDNBULL_Engine::$settings['included_file_extensions'] ) ?></textarea>
                                    <p class="description"><?php esc_html_e( 'Files with these extensions will be served by the CDN. One file extension per line.', 'cdn-bull' ); ?></p>
                                </label>
                            </fieldset>
                        </td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">
                            <?php esc_html_e( 'CDN Exclusions', 'cdn-bull' ); ?>
                        </th>
                        <td>
                            <fieldset>
                                <p class="subheading"><?php esc_html_e( 'Strings', 'cdn-bull' ); ?></p>
                                <label for="cdnbull_excluded_strings">
                                    <textarea name="cdnbull_settings[excluded_strings]" type="text" id="cdnbull_excluded_strings" class="regular-text code" rows="5" cols="40"><?php echo esc_textarea( CDNBULL_Engine::$settings['excluded_strings'] ) ?></textarea>
                                    <p class="description"><?php esc_html_e( 'URLs containing these strings will not be served by the CDN. One string per line.', 'cdn-bull' ) ?></p>
                                </label>
                            </fieldset>
                        </td>
                    </tr>
                </table>
                <h2 class="title">Purge CDN Cache</h2>
                <p><?php esc_html_e( 'You can purge the CDN cache from the WordPress admin bar.', 'cdn-bull' ) ?></p>
                <table class="form-table">
	                   <tr valign="top">
	                       <th scope="row">
	                           <?php esc_html_e("CDN Bull API Key", "cdn-bull"); ?>
	                       </th>
	                       <td>
	                           <fieldset>
	                               <label for="cdnbull_verify_key">
	                                   <input type="text" name="cdnbull_settings[cdnbull_verify_key]" id="cdnbull_verify_key" value="<?php echo esc_html( $options['cdnbull_verify_key'] ); ?>" size="64" class="regular-text code" />
											<?php
											if(isset($options['endpoint']) && empty($options['endpoint']) && empty($options['key_confirm'])	){

												echo '<a class="button button-primary" href="'.esc_url( CDNBULL_URL ).'register" target="_blank">
													'.esc_html__("GET NEW KEY", "cdn-bull").'
													 </a>';
											}

											if(!empty( $options['last_error'] )){
												echo '<div class="notice notice-error is-dismissible"><p>'.esc_html( $options['last_error'] ).'</p></div>';
											}
											?>
										<p class="description">
											<?php esc_html_e('CDN Bull API key.', 'cdn-bull' ); ?>
										</p>
	                               </label>
	                           </fieldset>
	                       </td>
	                   </tr>
	                   <tr valign="top">
	                       <th scope="row">
	                           <?php esc_html_e("Uninstall CDN Bull", "cdn-bull"); ?>
	                       </th>
	                       <td>
	                           <fieldset>
	                               <label for="cdnbull_del_settings">
	                                   <input type="checkbox" name="cdnbull_settings[del_settings]" id="cdnbull_del_settings" value="1" <?php checked(1, esc_html( $options['del_settings'] )) ?> />
	                                   <?php esc_html_e("Remove ALL settings.", "cdn-bull"); ?>
		                               <p class="description">
		                                   <?php esc_html_e("Check this if you would like to remove ALL CDN Bull data upon plugin deletion. All settings will be unrecoverable.", "cdn-bull"); ?>
		                               </p>
	                               </label>
	                           </fieldset>
	                       </td>
	                   </tr>
                </table>
                <p class="submit">
                    <input name="cdnbull_settings[validate_config]" type="submit" class="button-primary" value="<?php esc_html_e( 'Save Changes and Validate Configuration', 'cdn-bull' ); ?>" />
                </p>
            </form>
		<?php
		}
		if ( $tab == 'synct' ) {

			$backup = CDNBULL_Backups::get_instance()->get_status();
			$transient = get_transient( 'cdnbull_config_uploading_' . get_current_user_id() );
		?>
					<h1><?php esc_html_e( 'Sync Status', 'cdn-bull' ); ?></h1>
					<p class="cdnbull_messages">
						<?php
							if(isset($options['endpoint']) && empty($options['endpoint']) && empty($options['key_confirm'])	){

						                echo '<div class="notice notice-error is-dismissible"><p class="response">';
						                    esc_html_e( 'CDN Bull is almost ready. you need to enter your ', 'cdn-bull' ); ?>
						                    <a href="<?php echo esc_url( CDNBULL_URL ); ?>" target="_blank">
						                    	<?php esc_html_e( 'CDN Bull', 'cdn-bull' ); ?>
						                    </a> <?php esc_html_e( 'site key below. ', 'cdn-bull' ); ?>
						                    <?php
							                    printf(
									        		/* translators: 1: CDN Bull URL*/
										            wp_kses('Once you added your key, you can start to manage your site on <a href="%s" target="_blank">cdnbull.com</a>', 'cdn-bull' ),
										            esc_url( CDNBULL_URL . 'register' )
										        );
										     ?>
						                    <br>
						                    <?php esc_html_e( 'If you didn\'t create any site key for this site ', 'cdn-bull' ); ?>
						                    <a href="<?php echo esc_url( CDNBULL_URL ); ?>register" target="_blank">
						                    	<?php esc_html_e( 'Click here and get your site key.', 'cdn-bull' ); ?>
						                    </a>
						                    <?php esc_html_e( 'If you are using multisite WordPress, you need to enter site key for only main site.', 'cdn-bull' );
						                    echo '</p></div>';
								exit;
							}
									if(isset($backup) && !empty($backup) ){
                                        add_action( 'admin_footer', array( 'CDNBULL', 'check_sync_javascript' ) );
                                        /* translators: 1: CDN Bull settings URL*/
										echo sprintf(
											'<a href="%s" disabled class="disabled button button-primary large sync-button">%s</a>',
										    esc_url( wp_nonce_url( add_query_arg( ['page' => CDNBULL_PLUGIN_SLUG, 'tab' => $tab], admin_url( 'options-general.php' ), 'cdnbull_synct' ) ) ),
										    esc_html__(  'SYNC MY FILES', 'cdn-bull' )
                                        );
							               printf(
							                   '<div class="notice notice-error is-dismissible"><p class="response">%s</p></div>',
							                   wp_kses_post( $backup )
							               );

									} else if(isset($transient) && !empty($transient && $transient != 'No upload') ){
                                        /* translators: 1: CDN Bull settings URL*/
										echo sprintf(
											'<a href="%s" disabled class="disabled button button-primary large sync-button">%s</a>',
										    esc_url( wp_nonce_url( add_query_arg( ['page' => CDNBULL_PLUGIN_SLUG, 'tab' => $tab], admin_url('options-general.php'), 'cdnbull_synct') ) ),
										    esc_html__( 'SYNC MY FILES', 'cdn-bull' )
										);

                                        /* translators: 1: CDN Bull settings URL*/
										echo sprintf(
											'<a href="%s" class="button-primary large sync-button">%s</a>',
										    esc_url( wp_nonce_url( add_query_arg( ['page' => CDNBULL_PLUGIN_SLUG, 'tab' => $tab, '_cdn' => 'stopt'], admin_url('options-general.php'), 'cdnbull_synct') ) ),
										    esc_html__( 'STOP PROCESS', 'cdn-bull' )
										);

						                printf(
						                    wp_kses_post( $transient['wrapper'] ),
						                    wp_kses_post( $transient['message'] ),
						                );

									} else {
										/* translators: 1: CDN Bull settings URL*/

										echo sprintf(
										    '<a href="%s" class="button button-primary large sync-button" id="sync-button">%s</a>',
										    esc_url( wp_nonce_url( add_query_arg( ['page' => CDNBULL_PLUGIN_SLUG, 'tab' => $tab, '_cdn' => 'synct'], admin_url( 'options-general.php' ), 'cdnbull_backup_request' ) ) ),
										    esc_html__( 'SYNC MY FILES', 'cdn-bull' )
										);
									}

									if ( ! empty( $_GET['_cdn'] ) && sanitize_text_field( wp_unslash( $_GET['_cdn'] ) ) === 'upload' ) {

						                /* translators: 1: Please wait text*/
										printf(
										    '<div class="notice notice-error is-dismissible"><p class="response">%s</p></div>',
										    esc_html__( 'Please wait...', 'cdn-bull' )
										);

										add_action( 'admin_footer', array( 'CDNBULL', 'check_upload_javascript' ) );

									}


								if ( ! empty( $_GET['_cdn'] ) && sanitize_text_field( wp_unslash( $_GET['_cdn'] ) ) === 'stopt' ) {
					        		CDNBULL_Backups::get_instance()->cleanup_ziparchive_partials();
					        		delete_transient( 'cdnbull_last_backup_' . get_current_user_id() );

						               /* translators: 1: Please wait text*/
						               printf(
						                   '<div class="notice notice-error is-dismissible cdn-bull-response"><p class="response">%s</p></div>',
						                   esc_html__('Please wait...', 'cdn-bull' )
						               );

						               add_action( 'admin_footer', array( 'CDNBULL', 'stop_upload_javascript' ) );
								}
						?>
						</p>
		<?php
		}

		if ( $tab == 'support' ) { ?>
			<div  class="wrap" id="cdnbull_support">
						<p class="cdnbull_messages">
							<h2 style="margin-bottom: 0;"><span class="dashicons dashicons-format-chat" style="font-size: 32px; display: inline-table;"></span> <?php esc_html_e( 'Questions Achive ? ','cdn-bull' ); ?></h2>
							<?php esc_html_e( 'Before asking your question you can look or search our questions archive!.. ', 'cdn-bull' ); ?><br>
							<a href="<?php echo esc_url( CDNBULL_URL ); ?>faq" target="_blank">
									<?php esc_html_e( 'Take a look at Question Archive now!', 'cdn-bull' ); ?>
							</a>
							<br>
							<br>
							<h2 style="margin-bottom: 0;"><span class="dashicons dashicons-admin-users" style="font-size: 32px; display: inline-table;"></span> <?php esc_html_e( 'Find an Answer ? ', 'cdn-bull' ); ?></h2>
							<?php esc_html_e( 'You are the right section! you can find all your answer about CDN gfor WordPress service. ', 'cdn-bull' ); ?><br>
							<a href="<?php echo esc_url( CDNBULL_URL ); ?>support" target="_blank">
									<?php esc_html_e( 'Ask your question now!', 'cdn-bull' ); ?>
							</a>
						</p>
		<?php
		}

		if ( $tab == 'donate' ) { ?>
			<h1><?php esc_html_e( 'Donate', 'cdn-bull' ); ?></h1>
					<p class="cdnbull_messages">
						<h2 style="margin-bottom: 0;"><span class="dashicons dashicons-editor-help" style="font-size: 32px; display: inline-table;"></span> <?php esc_html_e( 'Do you like it? ', 'cdn-bull' ); ?></h2>
						<?php esc_html_e( 'Hi! I am Eric Simanko from the iRemoteWP developer team and one of the creators of this plugin. If you like this product that we spend a long time on, you can give us financial support. At least we can drink 1 coffee thanks to you :)', 'cdn-bull' ); ?><br>
						<br>
						<br>
						<div id="donate-button-container">
							<div id="donate-button"></div>
						</div>
					</p>
		<?php

			add_action( 'admin_footer', array( 'CDNBULL', 'paypal_donate_javascript' ) );
		}

		if ( $tab == 'hire_expert' ) { ?>
			<h1><?php esc_html_e( 'Hire an Expert', 'cdn-bull' ); ?></h1>
					<p class="cdnbull_messages">
						<h2 style="margin-bottom: 0;"><span class="dashicons dashicons-editor-help" style="font-size: 32px; display: inline-table;"></span> <?php esc_html_e( 'Need help? ', 'cdn-bull' ); ?></h2>
						<?php esc_html_e( 'If you have speed problems on your website or want to get support with CDN setup, you may want to work with our expert support team. <br>Please send an e-mail to get information about CDN setups, speed increase solutions and prices. <br><br>Thank you.
						', 'cdn-bull' ); ?>
						<br>
						<?php esc_html_e( 'Eric Simanko', 'cdn-bull' ); ?>
						<bR>
						<br>
						<?php esc_html_e( 'Mail adress', 'cdn-bull' ); ?>
						<a href="mailto:<?php esc_html_e( 'info@cdnbull.com', 'cdn-bull' ); ?>?subject=<?php esc_html_e( 'Hire an Expert', 'cdn-bull' ); ?>" target="_blank">
								<?php esc_html_e( 'info@cdnbull.com', 'cdn-bull' ); ?>
						</a>
					</p>
		<?php }

		?>
		</div>
		<?php
    }

}
